import inflect
from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html
from django_audit_fields.admin import audit_fieldset_tuple
from intecomm_form_validators import DISSOLVED, IN_FOLLOWUP

from intecomm_screening.admin.modeladmin_mixins import BaseModelAdminMixin

from ..admin_site import intecomm_group_admin
from ..forms import PatientGroupForm
from ..models import PatientGroup

p = inflect.engine()


@admin.register(PatientGroup, site=intecomm_group_admin)
class PatientGroupAdmin(BaseModelAdminMixin):

    """Modeladmin for patient groups in follow-up or dissolved.

    See `get_queryset`"""

    form = PatientGroupForm

    show_object_tools = True
    change_list_template: str = "intecomm_group/admin/patientgroup_change_list.html"
    show_save_next = False
    show_cancel = True

    fieldsets = (
        (
            None,
            {"fields": ("report_datetime", "name", "status", "notes")},
        ),
        audit_fieldset_tuple,
    )

    list_display = (
        "__str__",
        "opened",
        "status",
        "meetings",
        "patients",
        "user_created",
        "created",
    )

    list_filter = (
        "status",
        "randomized",
        "report_datetime",
        "randomized_datetime",
    )

    search_fields = (
        "name",
        "patients_hiv__legal_name__exact",
        "patients_hiv__familiar_name__exact",
        "patients_hiv__initials__iexact",
        "patients_dm__legal_name__exact",
        "patients_dm__familiar_name__exact",
        "patients_dm__initials__iexact",
        "patients_htn__legal_name__exact",
        "patients_htn__familiar_name__exact",
        "patients_htn__initials__iexact",
    )

    radio_fields = {
        "status": admin.VERTICAL,
    }

    readonly_fields = (
        "report_datetime",
        "name",
        "status",
        "randomized",
        "randomized_datetime",
    )

    @admin.display(description="Opened", ordering="report_datetime")
    def opened(self, obj=None):
        return obj.report_datetime.date()

    @admin.display(description="Meetings")
    def meetings(self, obj=None):
        name = "+".join(obj.name.split(" "))
        url = reverse("intecomm_prn_admin:intecomm_group_patientgroupmeeting_changelist")
        url = f"{url}?q={name}"
        return format_html(f'<a href="{url}">Meetings</a>')

    @admin.display(description="Patients")
    def patients(self, obj=None):
        cnt = (
            obj.hiv_patients.all().count()
            + obj.dm_patients.all().count()
            + obj.htn_patients.all().count()
        )
        url = reverse("intecomm_screening_admin:intecomm_screening_patientlog_changelist")
        url = f"{url}?q={obj.name}"
        return format_html(f'<a href="{url}">{cnt}&nbsp;{p.plural("patient", cnt)}</a>')

    def get_queryset(self, request):
        return super().get_queryset(request).filter(status__in=[IN_FOLLOWUP, DISSOLVED])



from intecomm_group.models import PatientGroup as BasePatientGroup


class PatientGroup(BasePatientGroup):
    def __str__(self):
        return self.name.upper()

    class Meta:
        proxy = True



from .health_facility_admin import HealthFacilityAdmin
from .health_talk_log_admin import HealthTalkLogAdmin
from .patient_call_admin import PatientCallAdmin
from .patient_group_admin import PatientGroupAdmin
from .patient_log_admin import PatientLogAdmin
from .site_admin import SiteAdmin
from .subject_refusal_admin import SubjectRefusalAdmin
from .subject_screening_admin import SubjectScreeningAdmin



from edc_auth.get_clinic_codenames import get_clinic_codenames

clinic_codenames = get_clinic_codenames(
    "intecomm_prn",
    "intecomm_group",
    "intecomm_subject",
    "intecomm_consent",
    "intecomm_screening",
    list_app="intecomm_lists",
)

screening_codenames = get_clinic_codenames(
    "intecomm_screening",
    list_app="intecomm_lists",
)
screening_codenames.sort()



from django.db import models
from django.utils.html import format_html
from edc_constants.choices import YES_NO, YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_dx_review.models import ReasonsForTesting
from edc_model.models import BaseUuidModel
from edc_model_fields.fields import OtherCharField
from edc_rx.model_mixins import TreatmentPayMethodsModelMixin

from ..model_mixins import ClinicalReviewModelMixin, CrfModelMixin


class ClinicalReview(
    TreatmentPayMethodsModelMixin,
    ClinicalReviewModelMixin,
    CrfModelMixin,
    BaseUuidModel,
):

    hiv_test = models.CharField(
        verbose_name="Since last seen, was the patient tested for HIV infection?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=format_html(
            "Note: Select `not applicable` if diagnosis previously reported. <BR>"
            "`Since last seen` includes today.<BR>"
            "If `yes', complete the initial review CRF<BR>"
            "If `not applicable`, complete the review CRF."
        ),
    )

    hiv_test_date = models.DateField(
        verbose_name="Date test requested",
        null=True,
        blank=True,
    )

    hiv_reason = models.ManyToManyField(
        ReasonsForTesting,
        related_name="hiv_test_reason",
        verbose_name="Why was the patient tested for HIV infection?",
        blank=True,
    )

    hiv_reason_other = OtherCharField()

    hiv_dx = models.CharField(
        verbose_name="As of today, was the patient newly diagnosed with HIV infection?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    htn_test = models.CharField(
        verbose_name="Since last seen, was the patient tested for hypertension?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=format_html(
            "Note: Select `not applicable` if diagnosis previously reported. <BR>"
            "`Since last seen` includes today.<BR>"
            "If `yes', complete the initial review CRF<BR>"
            "If `not applicable`, complete the review CRF."
        ),
    )

    htn_test_date = models.DateField(
        verbose_name="Date test requested",
        null=True,
        blank=True,
    )

    htn_reason = models.ManyToManyField(
        ReasonsForTesting,
        related_name="htn_test_reason",
        verbose_name="Why was the patient tested for hypertension?",
        blank=True,
    )

    htn_reason_other = OtherCharField()

    htn_dx = models.CharField(
        verbose_name="As of today, was the patient newly diagnosed with hypertension?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    dm_test = models.CharField(
        verbose_name="Since last seen, was the patient tested for diabetes?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=format_html(
            "Note: Select `not applicable` if diagnosis previously reported. <BR>"
            "`Since last seen` includes today.<BR>"
            "If `yes', complete the initial review CRF<BR>"
            "If `not applicable`, complete the review CRF."
        ),
    )

    dm_test_date = models.DateField(
        verbose_name="Date test requested",
        null=True,
        blank=True,
    )

    dm_reason = models.ManyToManyField(
        ReasonsForTesting,
        related_name="dm_reason",
        verbose_name="Why was the patient tested for diabetes?",
        blank=True,
    )

    dm_reason_other = OtherCharField()

    dm_dx = models.CharField(
        verbose_name="As of today, was the patient newly diagnosed with diabetes?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    complications = models.CharField(
        verbose_name="Since last seen, has the patient had any complications",
        max_length=15,
        choices=YES_NO,
        help_text="If Yes, complete the `Complications` CRF",
    )

    reason_other = OtherCharField()

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Clinical Review"
        verbose_name_plural = "Clinical Reviews"



from .end_of_study_form import EndOfStudyForm
from .loss_to_followup_form import LossToFollowupForm
from .offschedule_followup_form import OffScheduleFollowupForm
from .protocol_incident_form import ProtocolIncidentForm
from .subject_transfer_form import SubjectTransferForm



from dateutil.relativedelta import relativedelta
from edc_visit_schedule import Schedule
from edc_visit_schedule import Visit as BaseVisit

from ..constants import BASELINE_SCHEDULE, DAY1
from .crfs import crfs_d1, crfs_missed
from .crfs import crfs_prn as default_crfs_prn
from .crfs import crfs_unscheduled as default_crfs_unscheduled
from .requisitions import requisitions_d1
from .requisitions import requisitions_prn as default_requisitions_prn
from .requisitions import requisitions_unscheduled as default_requisitions_unscheduled


class Visit(BaseVisit):
    def __init__(
        self,
        crfs_unscheduled=None,
        requisitions_unscheduled=None,
        crfs_prn=None,
        requisitions_prn=None,
        allow_unscheduled=None,
        **kwargs,
    ):
        super().__init__(
            allow_unscheduled=True if allow_unscheduled is None else allow_unscheduled,
            crfs_unscheduled=crfs_unscheduled or default_crfs_unscheduled,
            requisitions_unscheduled=requisitions_unscheduled
            or default_requisitions_unscheduled,
            crfs_prn=crfs_prn or default_crfs_prn,
            requisitions_prn=requisitions_prn or default_requisitions_prn,
            crfs_missed=crfs_missed,
            **kwargs,
        )


baseline_schedule = Schedule(
    name=BASELINE_SCHEDULE,
    verbose_name="baseline",
    onschedule_model="intecomm_prn.onschedulebaseline",
    offschedule_model="intecomm_prn.offschedulebaseline",
    consent_model="intecomm_consent.subjectconsent",
    appointment_model="edc_appointment.appointment",
)


visit = Visit(
    code=DAY1,
    title="Day 1",
    timepoint=0,
    rbase=relativedelta(days=0),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=0),
    requisitions=requisitions_d1,
    crfs=crfs_d1,
    facility_name="7-day-clinic",
)


baseline_schedule.add_visit(visit=visit)



from edc_visit_schedule import VisitSchedule

from ..constants import VISIT_SCHEDULE
from .baseline_schedule import baseline_schedule

visit_schedule = VisitSchedule(
    name=VISIT_SCHEDULE,
    verbose_name="INTECOMM",
    offstudy_model="edc_offstudy.subjectoffstudy",
    death_report_model="intecomm_ae.deathreport",
    locator_model="edc_locator.subjectlocator",
    previous_visit_schedule=None,
)

visit_schedule.add_schedule(baseline_schedule)



from django.apps import apps as django_apps
from edc_metadata.metadata_rules import PredicateCollection
from edc_visit_schedule.utils import is_baseline


class Predicates(PredicateCollection):

    app_label = "intecomm_subject"
    visit_model = "intecomm_subject.subjectvisit"

    @staticmethod
    def family_history_required(visit, **kwargs):
        """Returns True if this is not the baseline visit
        and the CRF has NOT been previously completed.
        """
        required = False
        if not is_baseline(visit):
            model_cls = django_apps.get_model("intecomm_subject.familyhistory")
            if not model_cls.objects.filter(
                subject_visit__subject_identifier=visit.subject_identifier,
            ).exists():
                required = True
        return required



from .ae_initial_model_wrapper import AeInitialModelWrapper
from .death_report_model_wrapper import DeathReportModelWrapper
from .patient_log_model_wrapper import PatientLogModelWrapper
from .subject_refusal_model_wrapper import SubjectRefusalModelWrapper
from .subject_screening_model_wrapper import (
    SubjectConsentModelWrapper,
    SubjectScreeningModelWrapper,
)



from django.db import models
from edc_constants.choices import SMOKER_STATUS_SIMPLE, YES_NO
from edc_constants.constants import NOT_APPLICABLE
from edc_model import utils as edc_model_utils
from edc_model.models import BaseUuidModel, DurationYMDField
from edc_model_fields.fields import OtherCharField

from ..choices import ALCOHOL_CONSUMPTION, EDUCATION, EMPLOYMENT_STATUS, MARITAL_STATUS
from ..model_mixins import CrfModelMixin


class OtherBaselineData(CrfModelMixin, BaseUuidModel):

    employment_status = models.CharField(
        verbose_name="What is the patient's employment status?",
        max_length=25,
        choices=EMPLOYMENT_STATUS,
    )

    employment_status_other = OtherCharField()

    education = models.CharField(
        verbose_name="How much formal education does the patient have?",
        max_length=25,
        choices=EDUCATION,
    )

    marital_status = models.CharField(
        verbose_name="Personal status?",
        max_length=25,
        choices=MARITAL_STATUS,
    )

    smoking_status = models.CharField(
        verbose_name="Which of these options describes you",
        max_length=15,
        choices=SMOKER_STATUS_SIMPLE,
    )

    smoker_quit_ago = DurationYMDField(
        verbose_name="If you used to smoke but stopped, how long ago did you stop",
        null=True,
        blank=True,
    )

    smoker_quit_estimated_date = models.DateField(
        verbose_name="Estimated date smoker quit?",
        null=True,
        editable=False,
    )

    alcohol = models.CharField(
        verbose_name="Do you drink alcohol?",
        max_length=15,
        choices=YES_NO,
    )

    alcohol_consumption = models.CharField(
        verbose_name="If yes, how often do you drink alcohol?",
        max_length=25,
        choices=ALCOHOL_CONSUMPTION,
        default=NOT_APPLICABLE,
    )

    work_activity = models.CharField(
        verbose_name="Does your work involve moderate or vigorous-intensity activity that"
        "causes small to large increases in breathing or heart rate like "
        "carrying or lifting loads, digging or construction work",
        max_length=15,
        choices=YES_NO,
    )
    work_activity_days = models.IntegerField(
        verbose_name="In a typical week, how many days do you do these moderate or "
        "vigorous-intensity activities as part of your work?",
        blank=True,
        null=True,
        help_text="Give number of days in a 7-day week",
    )

    work_activity_exercise = models.IntegerField(
        verbose_name="In addition to the moderate or vigorous-intensity activity above, "
        "how many days do you do exercise, do sports or fitness activities in a "
        "typical week?",
        blank=True,
        null=True,
        help_text="Give number of days in a 7-day week",
    )
    work_activity_exercise_time = models.IntegerField(
        verbose_name="On an average day, how long do you spend combined on the moderate or "
        "vigorous-intensity activities and doing exercise, sports and fitness "
        "activities?",
        blank=True,
        null=True,
        help_text="Give number of minutes",
    )

    num_vegetables_eaten = models.IntegerField(
        verbose_name="On average, how many vegetables do you eat in a day?"
    )

    use_salt_on_food = models.CharField(
        verbose_name="Do you add salt to your food in addition to what has been added when "
        "cooking?",
        max_length=15,
        choices=YES_NO,
    )

    def save(self, *args, **kwargs):
        if self.smoker_quit_ago:
            self.smoker_quit_estimated_date = edc_model_utils.duration_to_date(
                self.smoker_quit_ago, self.report_datetime
            )
        super().save(*args, **kwargs)

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Other Baseline Data"
        verbose_name_plural = "Other Baseline Data"



from .health_facility import HealthFacility
from .health_talk_log import HealthTalkLog
from .patient_call import PatientCall
from .patient_log import PatientLog
from .proxy_models import PatientGroup, Site
from .signals import (
    patient_call_on_post_delete,
    patient_call_on_post_save,
    patient_group_on_post_save,
    update_subjectscreening_on_post_save,
)
from .subject_refusal import SubjectRefusal
from .subject_screening import SubjectScreening



# CLINICAL_WITHDRAWAL = "clinical_withdrawal"
# CLINICIAN = "clinician"
# DEATH_REPORT_ACTION = "submit_death_report"
# DEVIATION = "protocol_deviation"
# INVESTIGATOR = "investigator"
# LATE_EXCLUSION = "late_exclusion"
# LOSS_TO_FOLLOWUP_ACTION = "submit_loss_to_followup"
# OFFSCHEDULE_POSTNATAL_ACTION = "offschedule_postnatal_action"
# OFFSCHEDULE_PREGNANCY_ACTION = "offschedule_pregnancy_action"
# OFFSTUDY_MEDICATION_ACTION = "withdrawal_study_medication_action"
# OTHER_RX_DISCONTINUATION = "other_treatment_discontinuation"
# PREGNANCY_NOTIFICATION_ACTION = "pregancy_notification_action"
# PROTOCOL_DEVIATION_VIOLATION_ACTION = "submit-protocol-deviation-violation"
# SAE = "sae"
# UNBLINDING_REQUEST_ACTION = "unblinding-request"
# UNBLINDING_REVIEW_ACTION = "unblinding-review"
# VIOLATION = "protocol_violation"



from django import forms
from edc_form_validators import FormValidatorMixin
from intecomm_form_validators import PatientGroupScreeningFormValidator

from ..models import PatientGroup


class PatientGroupForm(FormValidatorMixin, forms.ModelForm):

    form_validator_cls = PatientGroupScreeningFormValidator

    class Meta:
        model = PatientGroup
        fields = "__all__"



from uuid import uuid4

from django.test import TestCase, tag
from edc_constants.constants import DM, HIV, HTN, NO, UUID_PATTERN
from intecomm_form_validators import RECRUITING
from model_bakery.baker import make_recipe

from intecomm_lists.models import Conditions
from intecomm_screening.models import PatientLog


class TestScreening(TestCase):
    def test_no_patients_ok(self):
        obj = make_recipe("intecomm_screening.patientgroup")
        self.assertEqual(obj.status, RECRUITING)
        self.assertIsNotNone(obj.name)
        self.assertIsNotNone(obj.group_identifier)
        self.assertRegexpMatches(str(obj.group_identifier), UUID_PATTERN)
        self.assertEqual(obj.ratio, None)
        self.assertEqual(obj.randomize_now, NO)
        self.assertFalse(obj.randomized)
        self.assertEqual(obj.patients.all().count(), 0)

    @tag("grp1")
    def test_with_patients_ok(self):
        initials = "ABCDEFGHIJKLMNOP"
        for i in range(0, 4):
            obj = make_recipe(
                "intecomm_screening.patientlog",
                legal_name=f"NAMEA{i} AAA{i}",
                familiar_name=f"NAMEA{i}",
                initials=f"N{initials[i]}A",
                hospital_identifier=uuid4().hex,
                contact_number=f"123456789{i}",
            )
            obj.conditions.add(Conditions.objects.get(name=HIV))
        for i in range(0, 5):
            make_recipe(
                "intecomm_screening.patientlog",
                legal_name=f"NAMEB{i} BBB{i}",
                familiar_name=f"NAMEB{i}",
                hospital_identifier=uuid4().hex,
                contact_number=f"12345678{i}9",
            )
            obj.conditions.add(Conditions.objects.get(name=DM))
        for i in range(0, 5):
            make_recipe(
                "intecomm_screening.patientlog",
                legal_name=f"NAMEC{i} CCC{i}",
                familiar_name=f"NAMEC{i}",
                hospital_identifier=uuid4().hex,
                contact_number=f"1234567{i}89",
            )
            obj.conditions.add(Conditions.objects.get(name=HTN))
        for patient_log in PatientLog.objects.all():
            self.assertEqual(patient_log.patientgroup_set.all().count(), 0)
        self.assertEqual(PatientLog.objects.all().count(), 14)

        # create patient_group
        patient_group = make_recipe("intecomm_screening.patientgroup")

        # add patient_logs to patient_group
        for index, patient_log in enumerate(PatientLog.objects.all()):
            patient_group.patients.add(patient_log)
        for patient_log in PatientLog.objects.all():
            self.assertEqual(patient_log.patientgroup_set.all().count(), 1)
            self.assertEqual(patient_log.patientgroup_set.all().first(), patient_group)

        patient_group.refresh_from_db()

        # TODO: why does this fail?
        # self.assertEqual(patient_group.patients.count(), 14)



from typing import Tuple

from django.contrib import admin
from django.core.handlers.wsgi import WSGIRequest
from django.template.loader import render_to_string
from django.urls.base import reverse
from django.urls.exceptions import NoReverseMatch
from django.utils.html import format_html
from django.utils.translation import gettext as _
from django_audit_fields import audit_fieldset_tuple
from edc_constants.choices import GENDER
from edc_dashboard.url_names import url_names
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_screening.utils import format_reasons_ineligible
from edc_sites.modeladmin_mixins import SiteModelAdminMixin

from ..admin_site import intecomm_screening_admin
from ..forms import SubjectScreeningForm
from ..models import PatientLog, SubjectScreening


@admin.register(SubjectScreening, site=intecomm_screening_admin)
class SubjectScreeningAdmin(
    SiteModelAdminMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin
):

    form = SubjectScreeningForm
    list_per_page = 15
    post_url_on_delete_name = "screening_listboard_url"
    subject_listboard_url_name = "screening_listboard_url"

    subject_dashboard_url_name = "screening_listboard_url"

    change_list_template: str = "intecomm_screening/admin/subjectscreening_change_list.html"

    additional_instructions = (
        "Patients must meet ALL of the inclusion criteria and NONE of the "
        "exclusion criteria in order to proceed to the final screening stage"
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "report_datetime",
                    "site",
                    "patient_log",
                )
            },
        ),
        (
            "Demographics",
            {
                "description": (
                    "Please review carefully. If anything needs to be changed, do so on "
                    "the Patient Log and try again"
                ),
                "fields": (
                    "legal_name",
                    "familiar_name",
                    "initials",
                    "hospital_identifier",
                    "gender",
                    "age_in_years",
                ),
            },
        ),
        (
            "Health facility",
            {
                "fields": (
                    "in_care_6m",
                    "in_care_duration",
                )
            },
        ),
        (
            "HIV",
            {
                "fields": (
                    "hiv_dx",
                    "hiv_dx_6m",
                    "hiv_dx_ago",
                    "art_unchanged_3m",
                    "art_stable",
                    "art_adherent",
                )
            },
        ),
        (
            "Diabetes",
            {
                "fields": (
                    "dm_dx",
                    "dm_dx_6m",
                    "dm_dx_ago",
                    "dm_complications",
                )
            },
        ),
        (
            "Hypertension",
            {
                "fields": (
                    "htn_dx",
                    "htn_dx_6m",
                    "htn_dx_ago",
                    "htn_complications",
                )
            },
        ),
        (
            "Pregnancy",
            {"fields": ("pregnant",)},
        ),
        (
            "Other history",
            {
                "fields": (
                    "excluded_by_bp_history",
                    "excluded_by_gluc_history",
                    "requires_acute_care",
                )
            },
        ),
        (
            "Location",
            {
                "fields": (
                    "lives_nearby",
                    "staying_nearby_6",
                )
            },
        ),
        (
            "Blood pressure measurements",
            {
                "fields": (
                    "sys_blood_pressure_one",
                    "dia_blood_pressure_one",
                    "sys_blood_pressure_two",
                    "dia_blood_pressure_two",
                )
            },
        ),
        (
            "Other",
            {
                "fields": (
                    "consent_ability",
                    "unsuitable_for_study",
                    "reasons_unsuitable",
                    "unsuitable_agreed",
                )
            },
        ),
        (
            "Updates",
            {
                "classes": ("collapse",),
                "fields": (
                    "screening_identifier",
                    "eligible",
                    "eligibility_datetime",
                    "real_eligibility_datetime",
                    "subject_identifier",
                ),
            },
        ),
        audit_fieldset_tuple,
    )

    list_display = (
        "screening_identifier",
        "eligibility_status",
        "demographics",
        "reasons",
        "report_datetime",
        "user_created",
        "created",
    )

    list_filter = (
        "report_datetime",
        "gender",
        "eligible",
        "consented",
        "refused",
    )

    search_fields = (
        "screening_identifier",
        "subject_identifier",
        "hospital_identifier",
        "initials",
        "reasons_ineligible",
    )

    readonly_fields = (
        "screening_identifier",
        "eligible",
        "eligibility_datetime",
        "real_eligibility_datetime",
        "subject_identifier",
    )

    radio_fields = {
        "consent_ability": admin.VERTICAL,
        "dm_dx": admin.VERTICAL,
        "dm_dx_6m": admin.VERTICAL,
        "gender": admin.VERTICAL,
        "hiv_dx": admin.VERTICAL,
        "hiv_dx_6m": admin.VERTICAL,
        "htn_dx": admin.VERTICAL,
        "htn_dx_6m": admin.VERTICAL,
        "selection_method": admin.VERTICAL,
        "lives_nearby": admin.VERTICAL,
        "staying_nearby_6": admin.VERTICAL,
        "in_care_6m": admin.VERTICAL,
        "art_unchanged_3m": admin.VERTICAL,
        "art_stable": admin.VERTICAL,
        "art_adherent": admin.VERTICAL,
        "dm_complications": admin.VERTICAL,
        "htn_complications": admin.VERTICAL,
        "pregnant": admin.VERTICAL,
        "excluded_by_bp_history": admin.VERTICAL,
        "excluded_by_gluc_history": admin.VERTICAL,
        "requires_acute_care": admin.VERTICAL,
        "unsuitable_for_study": admin.VERTICAL,
        "unsuitable_agreed": admin.VERTICAL,
    }

    def post_url_on_delete_kwargs(self, request, obj):
        return {}

    def get_readonly_fields(self, request, obj=None) -> Tuple[str, ...]:
        readonly_fields = super().get_readonly_fields(request, obj=obj)
        if obj:
            readonly_fields = readonly_fields + ("patient_log",)
        return readonly_fields

    @staticmethod
    def demographics(obj=None):
        data = [
            f"{obj.get_gender_display()} {obj.age_in_years}yrs",
            f"Initials: {obj.initials.upper()}<BR>",
            f"Hospital ID: {obj.hospital_identifier}",
        ]
        return format_html("<BR>".join(data))

    def reasons(self, obj=None):
        if not obj.reasons_ineligible:
            return self.dashboard(obj)
        return format_reasons_ineligible(obj.reasons_ineligible)

    def eligibility_status(self, obj=None):
        return None
        # eligibility = ScreeningEligibility(update_model=False)
        # screening_listboard_url = reverse(
        #     url_names.get(self.subject_listboard_url_name), args=(obj.screening_identifier,)
        # )
        # context = dict(
        #     title=_("Go to screening listboard"),
        #     url=f"{screening_listboard_url}?q={obj.screening_identifier}",
        #     label="Screening",
        # )
        # button = render_to_string("dashboard_button.html", context=context)
        # return format_html(button + "<BR>" + eligibility.eligibility_status(add_urls=True))

    def dashboard(self, obj=None, label=None):
        try:
            url = reverse(
                self.get_subject_dashboard_url_name(),
                kwargs=self.get_subject_dashboard_url_kwargs(obj),
            )
        except NoReverseMatch:
            url = reverse(url_names.get("screening_listboard_url"), kwargs={})
            context = dict(
                title=_("Go to screening and consent"),
                url=f"{url}?q={obj.screening_identifier}",
                label=label,
            )
        else:
            context = dict(title=_("Go to subject dashboard"), url=url, label=label)
        return render_to_string("dashboard_button.html", context=context)

    def formfield_for_foreignkey(self, db_field, request: WSGIRequest, **kwargs):
        db = kwargs.get("using")
        if db_field.name == "patient_log":
            if request.GET.get("patient_log"):
                kwargs["queryset"] = PatientLog.objects.using(db).filter(
                    id__exact=request.GET.get("patient_log", 0)
                )
            else:
                kwargs["queryset"] = PatientLog.objects.none()
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def view_on_site(self, obj) -> str:
        return reverse(self.get_subject_listboard_url_name())

    def formfield_for_choice_field(self, db_field, request, **kwargs):
        if db_field.name == "gender":
            kwargs["choices"] = GENDER
        return super().formfield_for_choice_field(db_field, request, **kwargs)



from .end_of_study_admin import EndOfStudyAdmin
from .loss_to_followup_admin import LossToFollowupAdmin
from .offschedule_baseline_admin import OffScheduleBaselineAdmin
from .offschedule_followup_admin import OffScheduleFollowupAdmin
from .onschedule_baseline_admin import OnScheduleBaselineAdmin
from .onschedule_followup_admin import OnScheduleFollowupAdmin
from .protocol_incident_admin import ProtocolDeviationViolationAdmin
from .subject_transfer_admin import SubjectTransferAdmin



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple

from ..admin_site import intecomm_subject_admin
from ..models import OtherBaselineData
from .modeladmin_mixins import CrfModelAdmin


@admin.register(OtherBaselineData, site=intecomm_subject_admin)
class OtherBaselineDataAdmin(CrfModelAdmin):
    # form = OtherBaselineDataForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        ("Smoking", {"fields": ("smoking_status", "smoker_quit_ago")}),
        ("Alcohol", {"fields": ("alcohol", "alcohol_consumption")}),
        ("Diet", {"fields": ("num_vegetables_eaten", "use_salt_on_food")}),
        (
            "Physical activity",
            {
                "fields": (
                    "work_activity",
                    "work_activity_days",
                    "work_activity_exercise",
                    "work_activity_exercise_time",
                )
            },
        ),
        (
            "Other",
            {
                "fields": (
                    "employment_status",
                    "employment_status_other",
                    "education",
                    "marital_status",
                )
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "crf_status": admin.VERTICAL,
        "smoking_status": admin.VERTICAL,
        "alcohol": admin.VERTICAL,
        "alcohol_consumption": admin.VERTICAL,
        "employment_status": admin.VERTICAL,
        "education": admin.VERTICAL,
        "marital_status": admin.VERTICAL,
        "work_activity": admin.VERTICAL,
        "use_salt_on_food": admin.VERTICAL,
    }



from decimal import Decimal

import inflect
from django.apps import apps as django_apps
from django.contrib import admin
from django.db.models import Count, Q
from django.urls import reverse
from django.utils.html import format_html
from django_audit_fields.admin import audit_fieldset_tuple
from edc_constants.constants import COMPLETE, DM, HIV, HTN, NEW
from edc_utils.round_up import round_up
from intecomm_form_validators import RECRUITING
from intecomm_form_validators.utils import get_min_group_size_for_ratio

from intecomm_group.exceptions import PatientGroupNotRandomized
from intecomm_group.utils import (
    get_assignment_description_for_patient_group,
    verify_patient_group_ratio_raise,
)

from ..admin_site import intecomm_screening_admin
from ..forms import PatientGroupForm
from ..models import PatientGroup
from .modeladmin_mixins import BaseModelAdminMixin

p = inflect.engine()


@admin.register(PatientGroup, site=intecomm_screening_admin)
class PatientGroupAdmin(BaseModelAdminMixin):

    form = PatientGroupForm

    show_object_tools = True
    change_list_template: str = "intecomm_screening/admin/patientgroup_change_list.html"
    change_list_help = "Searches on encrypted data work on exact uppercase matches only"
    change_list_title = PatientGroup._meta.verbose_name_plural

    fieldsets = (
        (
            None,
            {"fields": ("report_datetime", "name")},
        ),
        (
            "Patients",
            {
                "fields": (
                    "hiv_patients",
                    "dm_patients",
                    "htn_patients",
                    "multi_patients",
                ),
            },
        ),
        (
            "Selected Patients",
            {
                "description": (
                    "Changes made above will be reflected here after the form is saved."
                ),
                "fields": ("patients",),
            },
        ),
        (
            "Notes",
            {"fields": ("notes",)},
        ),
        (
            "Status, Size, Ratio",
            {
                "description": format_html(
                    "Please consult with your study coordinator before you "
                    "choose to override the minimum group size and/or the ratio of "
                    "NCD to HIV patients."
                ),
                "fields": (
                    "status",
                    "bypass_group_size_min",
                    "bypass_group_ratio",
                ),
            },
        ),
        (
            "Randomize",
            {
                "description": format_html(
                    "Complete this section when the group is COMPLETE and ready to "
                    "RANDOMIZE. <BR>Please consult with your study coordinator before "
                    "randomizing a group that does not meet the minimum group size and/or "
                    "the ratio of NCD to HIV patients. <BR>"
                    "<B>Important: THIS STEP CANNOT BE UNDONE</B>"
                ),
                "fields": (
                    "randomize_now",
                    "confirm_randomize_now",
                ),
            },
        ),
        audit_fieldset_tuple,
    )

    list_display = (
        "__str__",
        "opened",
        "status",
        "rounded_ratio",
        "arm",
        "members",
        "user_created",
        "created",
    )

    list_filter = (
        "status",
        "randomized",
        "report_datetime",
    )

    search_fields = (
        "name",
        "hiv_patients__legal_name__exact",
        "hiv_patients__familiar_name__exact",
        "hiv_patients__initials__iexact",
        "dm_patients__legal_name__exact",
        "dm_patients__familiar_name__exact",
        "dm_patients__initials__iexact",
        "htn_patients__legal_name__exact",
        "htn_patients__familiar_name__exact",
        "htn_patients__initials__iexact",
    )

    filter_horizontal = (
        "hiv_patients",
        "dm_patients",
        "htn_patients",
        "multi_patients",
        "patients",
    )

    radio_fields = {
        "status": admin.VERTICAL,
        "randomize_now": admin.VERTICAL,
    }

    readonly_fields = ("patients",)

    @admin.display(description="Opened", ordering="report_datetime")
    def opened(self, obj=None):
        return obj.report_datetime.date()

    @admin.display(description="Patients")
    def members(self, obj=None):
        cnt = (
            obj.hiv_patients.all().count()
            + obj.dm_patients.all().count()
            + obj.htn_patients.all().count()
            + obj.multi_patients.all().count()
        )
        cnt1 = obj.patients.all().count()
        url = reverse("intecomm_screening_admin:intecomm_screening_patientlog_changelist")
        url = f"{url}?q={obj.name}"
        return format_html(
            f'<a href="{url}">{cnt}&nbsp;{p.plural("patient", cnt)}({cnt1})</a>'
        )

    @admin.display(description="Arm")
    def arm(self, obj=None):
        try:
            return get_assignment_description_for_patient_group(obj.group_identifier)
        except PatientGroupNotRandomized:
            return None

    @admin.display(description="NCD:HIV", ordering="ratio")
    def rounded_ratio(self, obj=None):
        patients = obj.hiv_patients.all() | obj.dm_patients.all() | obj.htn_patients.all()
        ncd, hiv, ratio = verify_patient_group_ratio_raise(patients)
        ratio_str = ""
        if patients.count() >= get_min_group_size_for_ratio():
            ratio = round_up(ratio or Decimal("0.00"), Decimal("2.00"))
            ratio_str = f" ({str(ratio)})"
        return f"{ncd}:{hiv}{ratio_str}"

    def get_queryset(self, request):
        return (
            super()
            .get_queryset(request)
            .filter(status__in=[NEW, RECRUITING, COMPLETE], randomized=False)
        )

    def get_form(self, request, obj=None, **kwargs):
        form = super().get_form(request, obj, **kwargs)
        if obj:
            patient_log_model_cls = django_apps.get_model("intecomm_screening.patientlog")
            conditions = [HIV, DM, HTN]
            for cond in conditions:
                q_lookup = Q(patientgroup__isnull=True)
                if obj:
                    q_lookup = q_lookup | Q(patientgroup__name=obj.name)
                q_lookup = q_lookup, Q(conditions__name=cond)
                form.base_fields[
                    f"{cond.lower()}_patients"
                ].queryset = patient_log_model_cls.objects.filter(*q_lookup).exclude(
                    Q(conditions__name__in=[c for c in conditions if c != cond])
                )

            # multimorbidity
            qs = (
                patient_log_model_cls.objects.filter(
                    Q(patientgroup__name=obj.name) | Q(patientgroup__isnull=True)
                )
                .values("id")
                .annotate(conditions_count=Count("conditions__name"))
            )
            multi_morbidity_pks = [o.get("id") for o in qs if o.get("conditions_count") > 1]
            form.base_fields["multi_patients"].queryset = patient_log_model_cls.objects.filter(
                pk__in=multi_morbidity_pks
            )

        return form



from django.contrib import admin

from ..admin_site import intecomm_prn_admin
from ..models import OnScheduleBaseline
from .onschedule_followup_admin import OnScheduleFollowupAdmin


@admin.register(OnScheduleBaseline, site=intecomm_prn_admin)
class OnScheduleBaselineAdmin(OnScheduleFollowupAdmin):

    pass



from .blood_results import (
    BloodResultsFbcForm,
    BloodResultsGluForm,
    BloodResultsHba1cForm,
    BloodResultsInsForm,
    BloodResultsLftForm,
    BloodResultsLipidForm,
    BloodResultsRftForm,
)
from .cd4_result_form import Cd4ResultForm
from .clinical_review_baseline import ClinicalReviewBaselineForm
from .clinical_review_form import ClinicalReviewForm
from .concomitant_medication_form import ConcomitantMedicationForm
from .dm_review_form import DmReviewForm
from .drug_refill_dm_form import DrugRefillDmForm
from .drug_refill_hiv_form import DrugRefillHivForm
from .drug_refill_htn_form import DrugRefillHtnForm
from .drug_supply_dm_form import DrugSupplyDmForm
from .drug_supply_hiv_form import DrugSupplyHivForm
from .drug_supply_htn_form import DrugSupplyHtnForm
from .hiv_review_form import HivReviewForm
from .htn_review_form import HtnReviewForm
from .malaria_test_form import MalariaTestForm
from .subject_requisition_form import SubjectRequisitionForm
from .subject_visit_form import SubjectVisitForm
from .subject_visit_missed_form import SubjectVisitMissedForm
from .urine_dipstick_test_form import UrineDipstickTestForm
from .urine_pregnancy_form import UrinePregnancyForm
from .viral_load_result_form import ViralLoadResultForm



from __future__ import annotations

from decimal import Decimal
from typing import Tuple

from edc_constants.constants import DM, HIV, HTN
from edc_randomization.utils import (
    SubjectNotRandomization,
    get_assignment_description_for_subject,
)

from .exceptions import PatientGroupNotRandomized


class PatientGroupRatioError(Exception):
    pass


def verify_patient_group_ratio_raise(
    patients, raise_on_outofrange=None
) -> Tuple[int, int, Decimal | None]:
    ncd = 0.0
    hiv = 0.0
    for patient_log in patients:
        if patient_log.conditions.filter(name__in=[DM, HTN]).exists():
            ncd += 1.0
        elif patient_log.conditions.filter(name__in=[HIV]).exists():
            hiv += 1.0
    if not ncd or not hiv:
        ratio = 0.0
    else:
        ratio = ncd / hiv
    if raise_on_outofrange and not (2.0 <= ratio <= 2.7):
        raise PatientGroupRatioError(
            f"Ratio NDC:HIV not met. Expected at least 2:1. Got {int(ncd)}:{int(hiv)}. "
            "Perhaps catch this in the form."
        )
    ncd = int(ncd)
    hiv = int(hiv)
    ratio = Decimal(ratio)
    return ncd, hiv, ratio


def get_assignment_description_for_patient_group(group_identifier: str | None) -> str:
    try:
        description = get_assignment_description_for_subject(
            group_identifier, randomizer_name="default"
        )
    except SubjectNotRandomization:
        raise PatientGroupNotRandomized("Group is not randomized")
    return description



from django.db import models
from edc_constants.choices import YES_NO
from edc_constants.constants import NOT_APPLICABLE, YES
from edc_dx_review.model_mixins.initial_review import InitialReviewModelMixin
from edc_glucose.model_mixins import GlucoseModelMixin
from edc_model import duration_to_date
from edc_model.models import BaseUuidModel, DurationYMDField

from ..choices import DM_MANAGEMENT
from ..model_mixins import CrfModelMixin


class DmInitialReview(
    InitialReviewModelMixin, GlucoseModelMixin, CrfModelMixin, BaseUuidModel
):

    managed_by = models.CharField(
        verbose_name="How is the patient's diabetes managed?",
        max_length=25,
        choices=DM_MANAGEMENT,
        default=NOT_APPLICABLE,
    )

    med_start_ago = DurationYMDField(
        verbose_name=(
            "If the patient is taking medicines for diabetes, "
            "how long have they been taking these?"
        ),
        null=True,
        blank=True,
    )

    med_start_estimated_date = models.DateField(
        verbose_name="Estimated medication start date",
        null=True,
        editable=False,
    )

    med_start_date_estimated = models.CharField(
        verbose_name="Was the medication start date estimated?",
        max_length=15,
        choices=YES_NO,
        default=YES,
        editable=False,
    )

    glucose_performed = models.CharField(
        verbose_name="Has the patient had their glucose measured in the last few months?",
        max_length=15,
        choices=YES_NO,
    )

    def save(self, *args, **kwargs):
        if self.med_start_ago:
            self.med_start_estimated_date = duration_to_date(
                self.med_start_ago, self.report_datetime
            )
        super().save(*args, **kwargs)

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Diabetes Initial Review"
        verbose_name_plural = "Diabetes Initial Reviews"



from django.contrib import admin
from django.utils.html import format_html
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple

from ..admin_site import intecomm_subject_admin
from ..forms import ClinicalReviewBaselineForm
from ..models import ClinicalReviewBaseline
from .modeladmin_mixins import CrfModelAdmin


@admin.register(ClinicalReviewBaseline, site=intecomm_subject_admin)
class ClinicalReviewBaselineAdmin(CrfModelAdmin):

    form = ClinicalReviewBaselineForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "HIV",
            {
                "description": format_html(
                    "<h5>Note for HIV(+) patients:</h5> If patient is HIV(+), estimate or "
                    "provide the date patient FIRST tested positive."
                ),
                "fields": ("hiv_test", "hiv_test_ago", "hiv_test_date", "hiv_dx"),
            },
        ),
        ("Diabetes", {"fields": ("dm_test", "dm_test_ago", "dm_test_date", "dm_dx")}),
        (
            "Hypertension",
            {"fields": ("htn_test", "htn_test_ago", "htn_test_date", "htn_dx")},
        ),
        ("Other", {"fields": ("health_insurance", "patient_club")}),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "crf_status": admin.VERTICAL,
        "dm_dx": admin.VERTICAL,
        "dm_test": admin.VERTICAL,
        "health_insurance": admin.VERTICAL,
        "hiv_dx": admin.VERTICAL,
        "hiv_test": admin.VERTICAL,
        "htn_dx": admin.VERTICAL,
        "htn_test": admin.VERTICAL,
        "patient_club": admin.VERTICAL,
    }



from dateutil.relativedelta import relativedelta
from edc_visit_schedule import Schedule
from edc_visit_schedule import Visit as BaseVisit
from edc_visit_schedule.constants import (
    MONTH1,
    MONTH2,
    MONTH3,
    MONTH4,
    MONTH5,
    MONTH6,
    MONTH7,
    MONTH8,
    MONTH9,
    MONTH10,
    MONTH11,
    MONTH12,
)

from ..constants import FOLLOWUP_SCHEDULE
from .crfs import (
    crfs_1m,
    crfs_2m,
    crfs_3m,
    crfs_4m,
    crfs_5m,
    crfs_6m,
    crfs_7m,
    crfs_8m,
    crfs_9m,
    crfs_10m,
    crfs_11m,
    crfs_12m,
    crfs_missed,
)
from .crfs import crfs_prn as default_crfs_prn
from .crfs import crfs_unscheduled as default_crfs_unscheduled
from .requisitions import (
    requisitions_1m,
    requisitions_2m,
    requisitions_3m,
    requisitions_4m,
    requisitions_5m,
    requisitions_6m,
    requisitions_7m,
    requisitions_8m,
    requisitions_9m,
    requisitions_10m,
    requisitions_11m,
    requisitions_12m,
)
from .requisitions import requisitions_prn as default_requisitions_prn
from .requisitions import requisitions_unscheduled as default_requisitions_unscheduled


class Visit(BaseVisit):
    def __init__(
        self,
        crfs_unscheduled=None,
        requisitions_unscheduled=None,
        crfs_prn=None,
        requisitions_prn=None,
        allow_unscheduled=None,
        **kwargs,
    ):
        super().__init__(
            allow_unscheduled=True if allow_unscheduled is None else allow_unscheduled,
            crfs_unscheduled=crfs_unscheduled or default_crfs_unscheduled,
            requisitions_unscheduled=requisitions_unscheduled
            or default_requisitions_unscheduled,
            crfs_prn=crfs_prn or default_crfs_prn,
            requisitions_prn=requisitions_prn or default_requisitions_prn,
            crfs_missed=crfs_missed,
            **kwargs,
        )


# schedule for new participants
followup_schedule = Schedule(
    name=FOLLOWUP_SCHEDULE,
    verbose_name="followup",
    onschedule_model="intecomm_prn.onschedule_followup",
    offschedule_model="intecomm_prn.offschedule_followup",
    consent_model="intecomm_consent.subjectconsent",
    appointment_model="edc_appointment.appointment",
)


visit01 = Visit(
    code=MONTH1,
    title="Month 1",
    timepoint=1,
    rbase=relativedelta(months=1),
    rlower=relativedelta(days=5),
    rupper=relativedelta(days=10),
    requisitions=requisitions_1m,
    crfs=crfs_1m,
    facility_name="5-day-clinic",
)

visit02 = Visit(
    code=MONTH2,
    title="Month 2",
    timepoint=2,
    rbase=relativedelta(months=2),
    rlower=relativedelta(days=5),
    rupper=relativedelta(days=10),
    requisitions=requisitions_2m,
    crfs=crfs_2m,
    facility_name="5-day-clinic",
)

visit03 = Visit(
    code=MONTH3,
    title="Month 3",
    timepoint=3,
    rbase=relativedelta(months=3),
    rlower=relativedelta(days=5),
    rupper=relativedelta(days=10),
    requisitions=requisitions_3m,
    crfs=crfs_3m,
    facility_name="5-day-clinic",
)

visit04 = Visit(
    code=MONTH4,
    title="Month 4",
    timepoint=4,
    rbase=relativedelta(months=4),
    rlower=relativedelta(days=5),
    rupper=relativedelta(days=10),
    requisitions=requisitions_4m,
    crfs=crfs_4m,
    facility_name="5-day-clinic",
)

visit05 = Visit(
    code=MONTH5,
    title="Month 5",
    timepoint=5,
    rbase=relativedelta(months=5),
    rlower=relativedelta(days=5),
    rupper=relativedelta(days=10),
    requisitions=requisitions_5m,
    crfs=crfs_5m,
    facility_name="5-day-clinic",
)

visit06 = Visit(
    code=MONTH6,
    title="Month 6",
    timepoint=6,
    rbase=relativedelta(months=6),
    rlower=relativedelta(days=5),
    rupper=relativedelta(days=10),
    requisitions=requisitions_6m,
    crfs=crfs_6m,
    facility_name="5-day-clinic",
)

visit07 = Visit(
    code=MONTH7,
    title="Month 7",
    timepoint=7,
    rbase=relativedelta(months=7),
    rlower=relativedelta(days=5),
    rupper=relativedelta(days=10),
    requisitions=requisitions_7m,
    crfs=crfs_7m,
    facility_name="5-day-clinic",
)

visit08 = Visit(
    code=MONTH8,
    title="Month 8",
    timepoint=8,
    rbase=relativedelta(months=8),
    rlower=relativedelta(days=5),
    rupper=relativedelta(days=10),
    requisitions=requisitions_8m,
    crfs=crfs_8m,
    facility_name="5-day-clinic",
)

visit09 = Visit(
    code=MONTH9,
    title="Month 9",
    timepoint=9,
    rbase=relativedelta(months=9),
    rlower=relativedelta(days=5),
    rupper=relativedelta(days=10),
    requisitions=requisitions_9m,
    crfs=crfs_9m,
    facility_name="5-day-clinic",
)

visit10 = Visit(
    code=MONTH10,
    title="Month 10",
    timepoint=10,
    rbase=relativedelta(months=10),
    rlower=relativedelta(days=5),
    rupper=relativedelta(days=10),
    requisitions=requisitions_10m,
    crfs=crfs_10m,
    facility_name="5-day-clinic",
)

visit11 = Visit(
    code=MONTH11,
    title="Month 11",
    timepoint=11,
    rbase=relativedelta(months=11),
    rlower=relativedelta(days=5),
    rupper=relativedelta(days=10),
    requisitions=requisitions_11m,
    crfs=crfs_11m,
    facility_name="5-day-clinic",
)

visit12 = Visit(
    code=MONTH12,
    title="Month 12",
    timepoint=12,
    rbase=relativedelta(months=12),
    rlower=relativedelta(days=5),
    rupper=relativedelta(days=10),
    requisitions=requisitions_12m,
    crfs=crfs_12m,
    facility_name="5-day-clinic",
)

visits = [
    visit01,
    visit02,
    visit03,
    visit04,
    visit05,
    visit06,
    visit07,
    visit08,
    visit09,
    visit10,
    visit11,
    visit12,
]
for visit in visits:
    followup_schedule.add_visit(visit=visit)



from django.core.exceptions import ValidationError
from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver
from edc_visit_schedule.site_visit_schedules import site_visit_schedules

from intecomm_screening.models import SubjectScreening
from intecomm_subject.models import SubjectVisit

from .subject_consent import SubjectConsent


@receiver(
    post_save,
    weak=False,
    sender=SubjectConsent,
    dispatch_uid="subject_consent_on_post_save",
)
def subject_consent_on_post_save(sender, instance, raw, created, **kwargs):
    if not raw:
        if created:
            subject_screening = SubjectScreening.objects.get(
                screening_identifier=instance.screening_identifier
            )
            subject_screening.subject_identifier = instance.subject_identifier
            subject_screening.consented = True
            subject_screening.save_base(update_fields=["subject_identifier", "consented"])

            subject_screening.patient_log.subject_identifier = instance.subject_identifier
            subject_screening.patient_log.consent_datetime = instance.consent_datetime
            subject_screening.patient_log.save_base(
                update_fields=["subject_identifier", "consent_datetime"]
            )
            # put subject on schedule
            _, schedule = site_visit_schedules.get_by_onschedule_model(
                "intecomm_prn.onschedulebaseline"
            )
            schedule.put_on_schedule(
                subject_identifier=instance.subject_identifier,
                onschedule_datetime=instance.consent_datetime,
            )


@receiver(
    post_delete,
    weak=False,
    sender=SubjectConsent,
    dispatch_uid="subject_consent_on_post_delete",
)
def subject_consent_on_post_delete(sender, instance, using, **kwargs):
    """Updates/Resets subject screening."""
    # don't allow if subject visits exist. This should be caught
    # in the ModelAdmin delete view
    if SubjectVisit.objects.filter(subject_identifier=instance.subject_identifier).exists():
        raise ValidationError("Unable to delete consent. Visit data exists.")

    _, schedule = site_visit_schedules.get_by_onschedule_model(
        "intecomm_prn.onschedulebaseline"
    )
    schedule.take_off_schedule(
        subject_identifier=instance.subject_identifier,
        offschedule_datetime=instance.consent_datetime,
    )

    # update subject screening
    subject_screening = SubjectScreening.objects.get(
        screening_identifier=instance.screening_identifier
    )
    subject_screening.consented = False
    subject_screening.subject_identifier = subject_screening.subject_screening_as_pk
    subject_screening.save()



from edc_constants.constants import (
    CLINICAL_WITHDRAWAL,
    DEAD,
    DM,
    HIV,
    HTN,
    NCD,
    NOT_APPLICABLE,
    OTHER,
    PREGNANCY,
)
from edc_ltfu.constants import LOST_TO_FOLLOWUP
from edc_offstudy.constants import COMPLETED_FOLLOWUP, CONSENT_WITHDRAWAL
from edc_transfer.constants import TRANSFERRED

from intecomm_subject.constants import INTEGRATED

list_data = {
    "intecomm_lists.offstudyreasons": [
        (COMPLETED_FOLLOWUP, "Patient completed 12 months of follow-up"),
        (CONSENT_WITHDRAWAL, "Patient withdrew consent to participate further"),
        (CLINICAL_WITHDRAWAL, "Patient is withdrawn on CLINICAL grounds ..."),
        (LOST_TO_FOLLOWUP, "Patient lost to follow-up (at 12m only)"),
        (
            TRANSFERRED,
            "Patient has been transferred to another health centre/ area (at 12m only)",
        ),
        (DEAD, "Patient reported/known to have died"),
        (
            OTHER,
            "Other reason (specify below)",
        ),
    ],
    "intecomm_lists.subjectvisitmissedreasons": [
        ("forgot", "Forgot / Can’t remember being told about appointment"),
        ("family_emergency", "Family emergency (e.g. funeral) and was away"),
        ("travelling", "Away travelling/visiting"),
        ("working_schooling", "Away working/schooling"),
        ("too_sick", "Too sick or weak to come to the centre"),
        ("lack_of_transport", "Transportation difficulty"),
        (OTHER, "Other reason (specify below)"),
    ],
    "intecomm_lists.clinicalwithdrawalreasons": [
        (PREGNANCY, "Pregnancy"),
        ("kidney_disease", "Development of chronic kidney disease"),
        ("liver_disease", "Development of chronic liver disease"),
        ("intercurrent_illness", "Intercurrent illness which prevents further treatment"),
        ("investigator_decision", "Investigator decision"),
        (NOT_APPLICABLE, "Not applicable"),
        (
            OTHER,
            (
                "Other condition that justifies the discontinuation of "
                "treatment in the clinician’s opinion (specify below)"
            ),
        ),
    ],
    "intecomm_lists.healthtalktypes": [
        ("study_intro_talk", "INTECOMM study introduction"),
        ("sensitisation_talk", "INTECOMM Sensitisation"),
        (OTHER, "Other (specify below)"),
    ],
    "intecomm_lists.conditions": [
        (HIV, "HIV"),
        (DM, "Diabetes"),
        (HTN, "Hypertension"),
    ],
    "intecomm_lists.healthfacilitytypes": [
        (DM, "Diabetes Clinic"),
        (HIV, "HIV Clinic"),
        (HTN, "Hypertension Clinic"),
        (INTEGRATED, "Integrated Clinic (HIV/NCD)"),
        (NCD, "NCD Clinic"),
        (OTHER, "Other (specify below)"),
    ],
    "intecomm_lists.locationtypes": [
        ("school", "School"),
        (OTHER, "Other (specify below)"),
    ],
}



from django.contrib import admin

from ..admin_site import intecomm_prn_admin
from ..forms.offschedule_baseline_form import OffScheduleBaselineForm
from ..models import OffScheduleBaseline
from .offschedule_followup_admin import OffScheduleFollowupAdmin


@admin.register(OffScheduleBaseline, site=intecomm_prn_admin)
class OffScheduleBaselineAdmin(OffScheduleFollowupAdmin):

    form = OffScheduleBaselineForm



from .end_of_study import EndOfStudy
from .loss_to_followup import LossToFollowup
from .offschedule_baseline import OffScheduleBaseline
from .offschedule_followup import OffScheduleFollowup
from .onschedule_baseline import OnScheduleBaseline
from .onschedule_followup import OnScheduleFollowup
from .protocol_incident import ProtocolIncident
from .subject_transfer import SubjectTransfer
